@@ -217,7 +217,7 @@ If you create your Request object as one of the subclasses of Request, as in
 then it is a "canonical" request. You supply common parameter names and values (as in 'skills' => 'Perl')
 and the Request::Jobs class translates each parameter to the name and value expected by the target search engine.
 
-If you do not create a Request object (you just let Search::Scraper create one), as in
+If you do not create a Request object (you just let Scraper create one), as in
 
     $request = new WWW::Scraper('Google', { . . . });
 
@@ -313,12 +313,12 @@ There are three ways to implement this end.
 
 Apple Computer and Mozdev have established a standard format for accessing search engines named "Sherlock".
 Via a "plugin" that describes the search engine's submission form and results pages, both Mac's and Mozilla
-can access many search engines via a common interface. The Perl package C<WWW::Search::Sherlock> can read Sherlock
+can access many search engines via a common interface. The Perl package C<WWW::Scraper::Sherlock> can read Sherlock
 plugins and access those sites the same way.
 
 This is simplest way to get up and running on a search engine. You just name the plugin, provide your query,
 and watch it fly! You do not even need a sub-module associated with the search engine for this approach.
-See C<WWW::Search::Sherlock> for examples of how this is done.
+See C<WWW::Scraper::Sherlock> for examples of how this is done.
 
 There are about a hundred plugins available at F<http://sherlock.mozdev.org/source/browse/sherlock/www/>,
 contributed by many in the Open Source community.
@@ -336,7 +336,7 @@ Obviously, you are limited in this approach to those search engines that have pl
 
 You can supply the value for only one of the
 fields in the submission form. This still makes Sherlock pretty valuable, since it's rare that a search engine
-will have more than one interesting field to fill in (and WWW::Search::Sherlock will allow you to set other field
+will have more than one interesting field to fill in (and WWW::Scraper::Sherlock will allow you to set other field
 values, even though that's not in the Sherlock standard.)
 
 =item The Sherlock standard does not parse all the interesting data from the response.
@@ -575,7 +575,7 @@ parsing step finds this justification, then the parsing will stop. TD, DATA, REG
 but if you are using CALLBACK, you might not need the other data harvesting operations at all, so your
 callback subroutine must "declare" this justification when appropriate).
 
-See the code for C<WWW::Search::Sherlock> for an illustration of how this works. Sherlock uses this
+See the code for C<WWW::Scraper::Sherlock> for an illustration of how this works. Sherlock uses this
 method for almost all its parsing. A sample Sherlock scraper frame is also listed below in the EXAMPLES.
 
 =item TRYAGAIN
@@ -808,7 +808,7 @@ We'll leave this as an exercise for the reader (note that this is the "brief" fo
 =item Sherlock
 
 Now that you're fairly well-versed in Scraper frame operations syntax, we'll use a fairly complex frame,
-automatically generated by C<WWW::Search::Sherlock>, to illustrate how Sherlock uses the Scraper framework.
+automatically generated by C<WWW::Scraper::Sherlock>, to illustrate how Sherlock uses the Scraper framework.
 
 If you point Sherlock to the Yahoo plugin, it will generate the following Scraper frame to parse the result page.
 
@@ -893,11 +893,129 @@ Note that the start string for the second resultList is the same as the end stri
 This is but one illustration of how Sherlock handles things differently than Scraper. But by using the
 CALLBACK operation, just about any type of special treatment can be created for Scraper.
 
-We refer you to the code for C<WWW::Search::Sherlock> for further education on how to compose your own CALLBACK functions.
+We refer you to the code for C<WWW::Scraper::Sherlock> for further education on how to compose your own CALLBACK functions.
 
 =back
 
-=head1 DEPENDENCIES
+=head1 EXTENDING SCRAPER WITH NEW OPERATIONS
+
+You can extend Scraper by building new search engine interfaces,
+but there is a deeper meaning of extensibility that you can also apply.
+You can invent and use your own scraping operations in the Scraper frames themselves.
+
+By building a subclass of C<WWW::Scraper::Opcodecode>, you can use your invented operation by inserting your opcode into the scraper frame.
+Your code will be called to scrape the (local) HTML code, and put whatever results it garners into the Response object.
+You can even parameterize your opcode so you can modulate its behavior depending on where in the frame it appears.
+
+=head2 Example
+
+The first operation we implemented using Extensible Scraper Operations was 'GRUB'.
+GRUB is also one of the simpler operations, so it makes a good example.
+(Recall, from the description above, that GRUB scrapes all detail pages linked to the root detail page by 'next' links.)
+
+You will find this example code in F<WWW::Scraper::Opcodecode::GRUB.pm>. 
+See F<WWW::Scraper::Grub.pm> for how this is used, and F<eg/GrabGrub.pl> for a fully functional program using GRUB.
+
+The new() method instantiates the object and returns an array naming the fields that this operation captures.
+In the case of GRUB, it captures no newly named fields (it just appends new results to existing fields).
+
+Then the scrape() method is handed the (local) HTML code that it should examine.
+F<GRUB.pm> simply looks to a field that has already been captured in an earlier step,
+converts that to an absolute URL, then scrapes that page using ScrapeDetailPage().
+Recursion into ScrapeDetailPage() simply appends new field values to the existing ones
+(this is the normal behavior of ScrapeDetailPage().)
+F<GRUB.pm> also contains some code to control recursion, and to use multiple fields as the 'next' links.
+
+=head2 Methods
+
+=over 6
+
+=item Opcode::new ($scaffold, $params)
+
+The new() method should instantiate the object, and place a reference to an array of field names that this operation
+will capture into $self->{'fieldsDiscovered'}.
+If no fields are captured, then the hash-element should remain undefined.
+
+It should return itself.
+
+Example: (from Opcode::FORM.pm)
+
+    sub new {
+        my ($cls, $scaffold, $params) = @_;
+        return bless { 'fieldsDiscovered' => ['name','action','method'] };
+    }
+
+
+The base class (default) of new() will assume that all non-references in the scaffold are names of captured fields
+(see F<WWW::Scraper::Opcodecode>).
+
+=over 6
+
+=item $scaffold
+
+Points to the array is the scaffold element that invokes this extended operation.
+(The term "scaffold" refers to a particular location in the Scraper frame.)
+
+=item $params
+
+A reference to an array of strings that are the parameters of the operation,
+as specified within parenthesis immediately following the opcode in the scaffold.
+(A simple C<split> on commas is used to seperate these parameters into separate array elements.)
+
+This should not be confused with the subsequent scaffold parameters that appear after the opcode and its parameters.
+
+=back
+
+=item Opcode::scrape ($scraper, $scaffold, $TidyXML, $hit)
+
+The scrape() method is where the actual scraping takes place.
+
+This method should return undef if all it does is capture field values.
+It should return an array if it has deliniated a region to be processed by the sub-scaffold.
+That array should contain ($nextScaffold, $newString). 
+$newString should be undef if there was no change from the old string (harmless but wasteful if you return the old string).
+$newString should be an empty string if there is nothing left in the content to process.
+
+=over 6
+
+=item $scraper
+
+The Scraper interface object for this operation.
+
+=item $scaffold
+
+The scaffold is passed in here, again.
+
+=item $TidyXML
+
+A TidyXML object representing the current state of the scrape.
+(All scraping modes, not just TidyXML, are represented by a TidyXML object).
+$TidyXML->asString() will return the current (local) HTML code.
+You can replace that (after, for instance, capturing some data from the code that you don't want to see in the next iteration)
+by using $TidyXML->asString($newString) as a set method.
+
+=item $hit
+
+The current Response object. This is created when Scraper encounters the 'HIT' or 'HIT*' opcodes.
+Append newly scraped field values using the method $hit->fieldName($newFieldValue) .
+
+Example: (from Opcode::FORM.pm)
+
+    sub scrape {
+        my ($self, $scraper, $scaffold, $TidyXML, $hit) = @_;
+        
+        my ($sub_string, $attributes) = $TidyXML->getMarkedTextAndAttributes('FORM');
+        return undef unless $attributes;
+    
+        return ($$scaffold[1], $sub_string, $attributes);
+    }
+
+
+=back
+
+=back
+
+=head1 Dependencies
 
 In addition to F<WWW::Search>, Scraper depends on these non-core modules in order to support the
 translation of requests from canonical forms to native forms.
